package cn.com.infosec.crypto.modes;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class PGPCFBBlockCipher implements BlockCipher {
   private byte[] IV;
   private byte[] FR;
   private byte[] FRE;
   private byte[] tmp;
   private BlockCipher cipher;
   private int count;
   private int blockSize;
   private boolean forEncryption;
   private boolean inlineIv;

   public PGPCFBBlockCipher(BlockCipher cipher, boolean inlineIv) {
      PGPCFBBlockCipher var10000 = this;
      PGPCFBBlockCipher var10001 = this;
      PGPCFBBlockCipher var10002 = this;
      PGPCFBBlockCipher var10003 = this;
      super();
      this.cipher = cipher;
      this.inlineIv = inlineIv;
      int var3;
      this.blockSize = var3 = cipher.getBlockSize();
      var10003.IV = new byte[var3];
      var10002.FR = new byte[var3];
      var10001.FRE = new byte[var3];
      var10000.tmp = new byte[var3];
   }

   private byte encryptByte(byte data, int blockOff) {
      return (byte)(this.FRE[blockOff] ^ data);
   }

   private int encryptBlockWithIV(byte[] in, int inOff, byte[] out, int outOff) {
      int var5;
      if (inOff + (var5 = this.blockSize) <= in.length) {
         if (outOff + var5 > out.length) {
            throw new DataLengthException("output buffer too short");
         } else {
            int var6;
            byte[] var9;
            byte[] var10;
            if ((var6 = this.count) == 0) {
               var9 = this.FR;
               var10 = this.FRE;
               this.cipher.processBlock(var9, 0, var10, 0);

               for(var5 = 0; var5 < (var6 = this.blockSize); ++var5) {
                  var6 = outOff + var5;
                  out[var6] = this.encryptByte(this.IV[var5], var5);
               }

               System.arraycopy(out, outOff, this.FR, 0, var6);
               var9 = this.FR;
               var10 = this.FRE;
               this.cipher.processBlock(var9, 0, var10, 0);
               var6 = outOff + (var5 = this.blockSize);
               out[var6] = this.encryptByte(this.IV[var5 - 2], 0);
               var6 = outOff + (var5 = this.blockSize) + 1;
               out[var6] = this.encryptByte(this.IV[var5 - 1], 1);
               var5 = outOff + 2;
               var10 = this.FR;
               int var7 = this.blockSize;
               System.arraycopy(out, var5, var10, 0, var7);
               var9 = this.FR;
               var10 = this.FRE;
               this.cipher.processBlock(var9, 0, var10, 0);

               for(var5 = 0; var5 < (var6 = this.blockSize); ++var5) {
                  var6 = outOff + var6 + 2 + var5;
                  out[var6] = this.encryptByte(in[inOff + var5], var5);
               }

               int in1 = outOff + var6 + 2;
               System.arraycopy(out, in1, this.FR, 0, var6);
               in1 = this.count;
               int var10000 = inOff = this.blockSize;
               this.count = in1 + inOff * 2 + 2;
               return var10000 * 2 + 2;
            } else {
               if (var6 >= var5 + 2) {
                  var9 = this.FR;
                  var10 = this.FRE;
                  this.cipher.processBlock(var9, 0, var10, 0);

                  for(var5 = 0; var5 < (var6 = this.blockSize); ++var5) {
                     var6 = outOff + var5;
                     out[var6] = this.encryptByte(in[inOff + var5], var5);
                  }

                  System.arraycopy(out, outOff, this.FR, 0, var6);
               }

               return this.blockSize;
            }
         }
      } else {
         throw new DataLengthException("input buffer too short");
      }
   }

   private int decryptBlockWithIV(byte[] in, int inOff, byte[] out, int outOff) {
      int var5;
      if (inOff + (var5 = this.blockSize) <= in.length) {
         if (outOff + var5 > out.length) {
            throw new DataLengthException("output buffer too short");
         } else {
            int var6;
            byte[] inOff1;
            if ((var6 = this.count) == 0) {
               for(int out1 = 0; out1 < this.blockSize; ++out1) {
                  this.FR[out1] = in[inOff + out1];
               }

               in = this.FR;
               inOff1 = this.FRE;
               this.cipher.processBlock(in, 0, inOff1, 0);
               this.count += this.blockSize;
               return 0;
            } else {
               int in1;
               if (var6 == var5) {
                  System.arraycopy(in, inOff, this.tmp, 0, var5);
                  byte[] var10002 = this.FR;
                  in1 = this.blockSize - 2;
                  System.arraycopy(var10002, 2, var10002, 0, in1);
                  byte[] var10001 = in = this.FR;
                  var5 = (inOff = this.blockSize) - 2;
                  byte[] var11;
                  var10002 = var11 = this.tmp;
                  in[var5] = var11[0];
                  --inOff;
                  var10001[inOff] = var10002[1];
                  this.cipher.processBlock(in, 0, this.FRE, 0);

                  for(in1 = 0; in1 < (inOff = this.blockSize) - 2; ++in1) {
                     inOff = outOff + in1;
                     out[inOff] = this.encryptByte(this.tmp[in1 + 2], in1);
                  }

                  byte[] var7 = this.FR;
                  in1 = inOff - 2;
                  System.arraycopy(this.tmp, 2, var7, 0, in1);
                  this.count += 2;
                  return this.blockSize - 2;
               } else {
                  if (var6 >= var5 + 2) {
                     System.arraycopy(in, inOff, this.tmp, 0, var5);
                     in1 = outOff + 0;
                     out[in1] = this.encryptByte(this.tmp[0], this.blockSize - 2);
                     in1 = outOff + 1;
                     out[in1] = this.encryptByte(this.tmp[1], this.blockSize - 1);
                     in = this.FR;
                     inOff = this.blockSize - 2;
                     System.arraycopy(this.tmp, 0, in, inOff, 2);
                     in = this.FR;
                     inOff1 = this.FRE;
                     this.cipher.processBlock(in, 0, inOff1, 0);

                     for(in1 = 0; in1 < (inOff = this.blockSize) - 2; ++in1) {
                        inOff = outOff + in1 + 2;
                        out[inOff] = this.encryptByte(this.tmp[in1 + 2], in1);
                     }

                     in = this.FR;
                     inOff -= 2;
                     System.arraycopy(this.tmp, 2, in, 0, inOff);
                  }

                  return this.blockSize;
               }
            }
         }
      } else {
         throw new DataLengthException("input buffer too short");
      }
   }

   private int encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5;
      if (inOff + (var5 = this.blockSize) > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + var5 > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         byte[] var8 = this.FR;
         byte[] var6 = this.FRE;
         this.cipher.processBlock(var8, 0, var6, 0);

         for(var5 = 0; var5 < this.blockSize; ++var5) {
            int var9 = outOff + var5;
            out[var9] = this.encryptByte(in[inOff + var5], var5);
         }

         for(int in1 = 0; in1 < (inOff = this.blockSize); ++in1) {
            this.FR[in1] = out[outOff + in1];
         }

         return inOff;
      }
   }

   private int decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5;
      if (inOff + (var5 = this.blockSize) > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + var5 > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         byte[] var8 = this.FR;
         byte[] var6 = this.FRE;
         this.cipher.processBlock(var8, 0, var6, 0);

         for(var5 = 0; var5 < this.blockSize; ++var5) {
            int var9 = outOff + var5;
            out[var9] = this.encryptByte(in[inOff + var5], var5);
         }

         for(int out1 = 0; out1 < (outOff = this.blockSize); ++out1) {
            this.FR[out1] = in[inOff + out1];
         }

         return outOff;
      }
   }

   public BlockCipher getUnderlyingCipher() {
      return this.cipher;
   }

   public String getAlgorithmName() {
      return this.inlineIv ? this.cipher.getAlgorithmName() + "/PGPCFBwithIV" : this.cipher.getAlgorithmName() + "/PGPCFB";
   }

   public int getBlockSize() {
      return this.cipher.getBlockSize();
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      if (this.inlineIv) {
         return this.forEncryption ? this.encryptBlockWithIV(in, inOff, out, outOff) : this.decryptBlockWithIV(in, inOff, out, outOff);
      } else {
         return this.forEncryption ? this.encryptBlock(in, inOff, out, outOff) : this.decryptBlock(in, inOff, out, outOff);
      }
   }

   public void reset() {
      this.count = 0;

      byte[] var2;
      for(int var1 = 0; var1 != (var2 = this.FR).length; ++var1) {
         if (this.inlineIv) {
            var2[var1] = 0;
         } else {
            var2[var1] = this.IV[var1];
         }
      }

      this.cipher.reset();
   }

   public void init(boolean forEncryption, CipherParameters params) {
      this.forEncryption = forEncryption;
      if (params instanceof ParametersWithIV) {
         byte[] var3;
         ParametersWithIV forEncryption1;
         byte[] params1;
         if ((params1 = (forEncryption1 = (ParametersWithIV)params).getIV()).length < (var3 = this.IV).length) {
            int var4 = var3.length - params1.length;
            int var5 = params1.length;
            System.arraycopy(params1, 0, var3, var4, var5);

            byte[] var10;
            for(int var9 = 0; var9 < (var10 = this.IV).length - params1.length; ++var9) {
               var10[var9] = 0;
            }
         } else {
            int params2 = var3.length;
            System.arraycopy(params1, 0, var3, 0, params2);
         }

         this.reset();
         this.cipher.init(true, forEncryption1.getParameters());
      } else {
         this.reset();
         this.cipher.init(true, params);
      }

   }
}
